/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package edu.uiowa.icts.jbpm.test;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jbpm.api.task.Task;
import org.jbpm.test.JbpmTestCase;


/**
 * @author Tom Baeyens
 */
public class SubProcessTest extends JbpmTestCase {

  String deploymentId;
  String reviewProcId;
  
	private static final Log log =LogFactory.getLog(SubProcessTest.class);

  protected void setUp() throws Exception {
    super.setUp();
    
    deploymentId = repositoryService.createDeployment()
        .addResourceFromClasspath("edu/uiowa/icts/jbpm/test/process.jpdl.xml")
        .deploy();
    
    reviewProcId = repositoryService.createDeployment()
    .addResourceFromClasspath("edu/uiowa/icts/jbpm/test/CRU-Review.jpdl.xml")
    .deploy();
  }

  protected void tearDown() throws Exception {
    repositoryService.deleteDeploymentCascade(deploymentId);
    repositoryService.deleteDeploymentCascade(reviewProcId);
    
    super.tearDown();
  }

  public void testTaskComments() {
      Map<String, Object> params = new HashMap<String, Object>();

      List<String> serviceList = new ArrayList<String>();
		serviceList.add( "schappetj" );

      params.put("services", serviceList.toArray());

      params.put("irbNumber", "20101212111");
      params.put("cru_admin", "schappetj");
      log.debug("Quorum Size: " + serviceList.size());
      
      params.put("quorum", serviceList.size());
      params.put("safety_reviewer", "schappetj");
      params.put("statitical_reviewer", "schappetj");
      params.put("cru_chair", "schappetj");
      params.put("applicaitonId",  "111");
    executionService.startProcessInstanceByKey("CRU-Approval-Process", params);

    List<Task> taskList = taskService.findPersonalTasks("schappetj");
    
    assertEquals(1, taskList.size());
    String taskId = taskList.get(0).getId();
    /*** 
     * 
     * Mark adminPreReview Complete 
     * 
     * ***/
    taskService.completeTask(taskId); 
    
    taskList = taskService.findPersonalTasks("schappetj");
    assertEquals(4, taskList.size());
    /**** At this point there are 4 tasks to be completed  ***/
    
    log.debug("taskList.size()=" + taskList.size());
    taskList= taskService.createTaskQuery().activityName("Review").list();
    
    
    assertEquals(1, taskList.size());
    
    taskId = taskList.get(0).getId();
    taskService.completeTask(taskId); 
    /*** 
     * 
     * When completing task, all of the other tasks are "obsoleted", and we move the adminPostReview task. 
     * There should be 3 other tasks to be completed.
     * 
     * ***/
    
    taskList = taskService.findPersonalTasks("schappetj");
    assertEquals(3, taskList.size());
  }
}
